package javax.swing.text.html;

import javax.swing.event.DocumentEvent;
import java.util.Iterator;
import javax.swing.text.Position;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Vector;
import javax.accessibility.AccessibleTable;
import javax.accessibility.AccessibleIcon;
import java.awt.Image;
import javax.swing.text.Segment;
import java.text.BreakIterator;
import javax.swing.text.StyledDocument;
import javax.swing.text.PlainDocument;
import java.awt.Container;
import javax.swing.text.BadLocationException;
import javax.accessibility.AccessibleText;
import java.awt.event.FocusListener;
import java.beans.PropertyChangeEvent;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.FontMetrics;
import java.awt.Font;
import java.awt.Cursor;
import java.awt.Color;
import java.awt.IllegalComponentStateException;
import java.util.Locale;
import javax.swing.text.JTextComponent;
import javax.accessibility.AccessibleState;
import javax.accessibility.AccessibleStateSet;
import javax.accessibility.AccessibleComponent;
import javax.accessibility.AccessibleRole;
import javax.accessibility.AccessibleContext;
import javax.swing.text.AttributeSet;
import javax.swing.text.StyleConstants;
import javax.swing.text.Element;
import javax.swing.text.AbstractDocument;
import java.awt.Insets;
import java.awt.Rectangle;
import javax.swing.text.View;
import java.beans.PropertyChangeListener;
import javax.swing.event.DocumentListener;
import javax.swing.text.Document;
import javax.swing.JEditorPane;
import javax.accessibility.Accessible;

class AccessibleHTML implements Accessible
{
    private JEditorPane editor;
    private Document model;
    private DocumentListener docListener;
    private PropertyChangeListener propChangeListener;
    private AccessibleHTML.ElementInfo rootElementInfo;
    private AccessibleHTML.RootHTMLAccessibleContext rootHTMLAccessibleContext;
    
    public AccessibleHTML(JEditorPane editor) {
        this.editor = editor;
        this.propChangeListener = new AccessibleHTML.PropertyChangeHandler();
        this.setDocument(this.editor.getDocument());
        this.docListener = new AccessibleHTML.DocumentHandler();
    }
    
    private void setDocument(Document model) {
        if (this.model != null) {
            this.model.removeDocumentListener(this.docListener);
        }
        if (this.editor != null) {
            this.editor.removePropertyChangeListener(this.propChangeListener);
        }
        this.model = model;
        if (this.model != null) {
            if (this.rootElementInfo != null) {
                this.rootElementInfo.invalidate(false);
            }
            this.buildInfo();
            this.model.addDocumentListener(this.docListener);
        }
        else {
            this.rootElementInfo = null;
        }
        if (this.editor != null) {
            this.editor.addPropertyChangeListener(this.propChangeListener);
        }
    }
    
    private Document getDocument() {
        return this.model;
    }
    
    private JEditorPane getTextComponent() {
        return this.editor;
    }
    
    private AccessibleHTML.ElementInfo getRootInfo() {
        return this.rootElementInfo;
    }
    
    private View getRootView() {
        return this.getTextComponent().getUI().getRootView(this.getTextComponent());
    }
    
    private Rectangle getRootEditorRect() {
        Rectangle bounds = this.getTextComponent().getBounds();
        if (bounds.width > 0 && bounds.height > 0) {
            Rectangle rectangle = bounds;
            int n = 0;
            bounds.y = n;
            rectangle.x = n;
            Insets insets = this.editor.getInsets();
            bounds.x += insets.left;
            bounds.y += insets.top;
            bounds.width -= insets.left + insets.right;
            bounds.height -= insets.top + insets.bottom;
            return bounds;
        }
        return null;
    }
    
    private Object lock() {
        Document document = this.getDocument();
        if ((document instanceof AbstractDocument)) {
            ((AbstractDocument)document).readLock();
            return document;
        }
        return null;
    }
    
    private void unlock(Object o) {
        if (o != null) {
            ((AbstractDocument)o).readUnlock();
        }
    }
    
    private void buildInfo() {
        Object lock = this.lock();
        try {
            (this.rootElementInfo = new AccessibleHTML.ElementInfo(this.getDocument().getDefaultRootElement())).validate();
        }
        finally {
            this.unlock(lock);
        }
    }
    
    AccessibleHTML.ElementInfo createElementInfo(Element element, AccessibleHTML.ElementInfo elementInfo) {
        AttributeSet attributes = element.getAttributes();
        if (attributes != null) {
            Object attribute = attributes.getAttribute(StyleConstants.NameAttribute);
            if (attribute == HTML.Tag.IMG) {
                return new AccessibleHTML.IconElementInfo(element, elementInfo);
            }
            if (attribute == HTML.Tag.CONTENT || attribute == HTML.Tag.CAPTION) {
                return new AccessibleHTML.TextElementInfo(element, elementInfo);
            }
            if (attribute == HTML.Tag.TABLE) {
                return new AccessibleHTML.TableElementInfo(element, elementInfo);
            }
        }
        return null;
    }
    
    @Override
    public AccessibleContext getAccessibleContext() {
        if (this.rootHTMLAccessibleContext == null) {
            this.rootHTMLAccessibleContext = new AccessibleHTML.RootHTMLAccessibleContext(this.rootElementInfo);
        }
        return this.rootHTMLAccessibleContext;
    }
    
    static JEditorPane access$400(AccessibleHTML accessibleHTML) {
        return accessibleHTML.getTextComponent();
    }
    
    static Object access$1300(AccessibleHTML accessibleHTML) {
        return accessibleHTML.lock();
    }
    
    static View access$1400(AccessibleHTML accessibleHTML) {
        return accessibleHTML.getRootView();
    }
    
    static void access$1500(AccessibleHTML accessibleHTML, Object o) {
        accessibleHTML.unlock(o);
    }
    
    static Rectangle access$1600(AccessibleHTML accessibleHTML) {
        return accessibleHTML.getRootEditorRect();
    }
    
    static AccessibleHTML.ElementInfo access$1700(AccessibleHTML accessibleHTML) {
        return accessibleHTML.getRootInfo();
    }
    
    static void access$1900(AccessibleHTML accessibleHTML, Document document) {
        accessibleHTML.setDocument(document);
    }
    
    private class RootHTMLAccessibleContext extends AccessibleHTML.HTMLAccessibleContext
    {
        public RootHTMLAccessibleContext(AccessibleHTML.ElementInfo elementInfo) {
            super(elementInfo);
        }
        
        @Override
        public String getAccessibleName() {
            if (AccessibleHTML.this.model != null) {
                return (String)AccessibleHTML.this.model.getProperty("title");
            }
            return null;
        }
        
        @Override
        public String getAccessibleDescription() {
            return AccessibleHTML.this.editor.getContentType();
        }
        
        @Override
        public AccessibleRole getAccessibleRole() {
            return AccessibleRole.TEXT;
        }
    }
    
    protected abstract class HTMLAccessibleContext extends AccessibleContext implements Accessible, AccessibleComponent
    {
        protected AccessibleHTML.ElementInfo elementInfo;
        
        public HTMLAccessibleContext(AccessibleHTML.ElementInfo elementInfo) {
            this.elementInfo = elementInfo;
        }
        
        @Override
        public AccessibleContext getAccessibleContext() {
            return this;
        }
        
        @Override
        public AccessibleStateSet getAccessibleStateSet() {
            AccessibleStateSet set = new AccessibleStateSet();
            JEditorPane access$400 = AccessibleHTML.access$400(AccessibleHTML.this);
            if (access$400.isEnabled()) {
                set.add(AccessibleState.ENABLED);
            }
            if (access$400 instanceof JTextComponent && ((JTextComponent)access$400).isEditable()) {
                set.add(AccessibleState.EDITABLE);
                set.add(AccessibleState.FOCUSABLE);
            }
            if (access$400.isVisible()) {
                set.add(AccessibleState.VISIBLE);
            }
            if (access$400.isShowing()) {
                set.add(AccessibleState.SHOWING);
                return set;
            }
            return set;
        }
        
        @Override
        public int getAccessibleIndexInParent() {
            return this.elementInfo.getIndexInParent();
        }
        
        @Override
        public int getAccessibleChildrenCount() {
            return this.elementInfo.getChildCount();
        }
        
        @Override
        public Accessible getAccessibleChild(int n) {
            AccessibleHTML.ElementInfo child = this.elementInfo.getChild(n);
            if (child != null && child instanceof Accessible) {
                return (Accessible)child;
            }
            return null;
        }
        
        @Override
        public Locale getLocale() throws IllegalComponentStateException {
            return AccessibleHTML.this.editor.getLocale();
        }
        
        @Override
        public AccessibleComponent getAccessibleComponent() {
            return this;
        }
        
        @Override
        public Color getBackground() {
            return AccessibleHTML.access$400(AccessibleHTML.this).getBackground();
        }
        
        @Override
        public void setBackground(Color background) {
            AccessibleHTML.access$400(AccessibleHTML.this).setBackground(background);
        }
        
        @Override
        public Color getForeground() {
            return AccessibleHTML.access$400(AccessibleHTML.this).getForeground();
        }
        
        @Override
        public void setForeground(Color foreground) {
            AccessibleHTML.access$400(AccessibleHTML.this).setForeground(foreground);
        }
        
        @Override
        public Cursor getCursor() {
            return AccessibleHTML.access$400(AccessibleHTML.this).getCursor();
        }
        
        @Override
        public void setCursor(Cursor cursor) {
            AccessibleHTML.access$400(AccessibleHTML.this).setCursor(cursor);
        }
        
        @Override
        public Font getFont() {
            return AccessibleHTML.access$400(AccessibleHTML.this).getFont();
        }
        
        @Override
        public void setFont(Font font) {
            AccessibleHTML.access$400(AccessibleHTML.this).setFont(font);
        }
        
        @Override
        public FontMetrics getFontMetrics(Font font) {
            return AccessibleHTML.access$400(AccessibleHTML.this).getFontMetrics(font);
        }
        
        @Override
        public boolean isEnabled() {
            return AccessibleHTML.access$400(AccessibleHTML.this).isEnabled();
        }
        
        @Override
        public void setEnabled(boolean enabled) {
            AccessibleHTML.access$400(AccessibleHTML.this).setEnabled(enabled);
        }
        
        @Override
        public boolean isVisible() {
            return AccessibleHTML.access$400(AccessibleHTML.this).isVisible();
        }
        
        @Override
        public void setVisible(boolean visible) {
            AccessibleHTML.access$400(AccessibleHTML.this).setVisible(visible);
        }
        
        @Override
        public boolean isShowing() {
            return AccessibleHTML.access$400(AccessibleHTML.this).isShowing();
        }
        
        @Override
        public boolean contains(Point point) {
            Rectangle bounds = this.getBounds();
            return bounds != null && bounds.contains(point.x, point.y);
        }
        
        @Override
        public Point getLocationOnScreen() {
            Point locationOnScreen = AccessibleHTML.access$400(AccessibleHTML.this).getLocationOnScreen();
            Rectangle bounds = this.getBounds();
            if (bounds != null) {
                return new Point(locationOnScreen.x + bounds.x, locationOnScreen.y + bounds.y);
            }
            return null;
        }
        
        @Override
        public Point getLocation() {
            Rectangle bounds = this.getBounds();
            if (bounds != null) {
                return new Point(bounds.x, bounds.y);
            }
            return null;
        }
        
        @Override
        public void setLocation(Point point) {
        }
        
        @Override
        public Rectangle getBounds() {
            return this.elementInfo.getBounds();
        }
        
        @Override
        public void setBounds(Rectangle rectangle) {
        }
        
        @Override
        public Dimension getSize() {
            Rectangle bounds = this.getBounds();
            if (bounds != null) {
                return new Dimension(bounds.width, bounds.height);
            }
            return null;
        }
        
        @Override
        public void setSize(Dimension size) {
            AccessibleHTML.access$400(AccessibleHTML.this).setSize(size);
        }
        
        @Override
        public Accessible getAccessibleAt(Point point) {
            AccessibleHTML.ElementInfo elementInfo = this.getElementInfoAt(AccessibleHTML.this.rootElementInfo, point);
            if ((elementInfo instanceof Accessible)) {
                return (Accessible)elementInfo;
            }
            return null;
        }
        
        private AccessibleHTML.ElementInfo getElementInfoAt(AccessibleHTML.ElementInfo elementInfo, Point point) {
            if (elementInfo.getBounds() == null) {
                return null;
            }
            if (elementInfo.getChildCount() == 0 && elementInfo.getBounds().contains(point)) {
                return elementInfo;
            }
            if ((elementInfo instanceof AccessibleHTML.TableElementInfo)) {
                AccessibleHTML.ElementInfo captionInfo = ((AccessibleHTML.TableElementInfo)elementInfo).getCaptionInfo();
                if (captionInfo != null) {
                    Rectangle bounds = captionInfo.getBounds();
                    if (bounds != null && bounds.contains(point)) {
                        return captionInfo;
                    }
                }
            }
            int i = 0;
            while (i < elementInfo.getChildCount()) {
                AccessibleHTML.ElementInfo elementInfo2 = this.getElementInfoAt(elementInfo.getChild(i), point);
                if (elementInfo2 != null) {
                    return elementInfo2;
                }
                ++i;
            }
            return null;
        }
        
        @Override
        public boolean isFocusTraversable() {
            JEditorPane access$400 = AccessibleHTML.access$400(AccessibleHTML.this);
            return access$400 instanceof JTextComponent && ((JTextComponent)access$400).isEditable();
        }
        
        @Override
        public void requestFocus() {
            if (!this.isFocusTraversable()) {
                return;
            }
            JEditorPane access$400 = AccessibleHTML.access$400(AccessibleHTML.this);
            if ((access$400 instanceof JTextComponent)) {
                access$400.requestFocusInWindow();
                try {
                    if (this.elementInfo.validateIfNecessary()) {
                        ((JTextComponent)access$400).setCaretPosition(this.elementInfo.getElement().getStartOffset());
                        AccessibleHTML.this.editor.getAccessibleContext().firePropertyChange("AccessibleState", null, new PropertyChangeEvent(this, "AccessibleState", null, AccessibleState.FOCUSED));
                    }
                }
                catch (IllegalArgumentException ex) {}
            }
            return;
        }
        
        @Override
        public void addFocusListener(FocusListener l) {
            AccessibleHTML.access$400(AccessibleHTML.this).addFocusListener(l);
        }
        
        @Override
        public void removeFocusListener(FocusListener l) {
            AccessibleHTML.access$400(AccessibleHTML.this).removeFocusListener(l);
        }
    }
    
    class TextElementInfo extends AccessibleHTML.ElementInfo implements Accessible
    {
        private AccessibleContext accessibleContext;
        
        TextElementInfo(Element element, AccessibleHTML.ElementInfo elementInfo) {
            super(element, elementInfo);
        }
        
        @Override
        public AccessibleContext getAccessibleContext() {
            if (this.accessibleContext == null) {
                this.accessibleContext = new AccessibleHTML.TextElementInfo.TextAccessibleContext(this);
            }
            return this.accessibleContext;
        }
        
        public class TextAccessibleContext extends AccessibleHTML.HTMLAccessibleContext implements AccessibleText
        {
            public TextAccessibleContext(AccessibleHTML.ElementInfo elementInfo) {
                AccessibleHTML.TextElementInfo.this.this$0.super(elementInfo);
            }
            
            @Override
            public AccessibleText getAccessibleText() {
                return this;
            }
            
            @Override
            public String getAccessibleName() {
                if (AccessibleHTML.this.model != null) {
                    return (String)AccessibleHTML.this.model.getProperty("title");
                }
                return null;
            }
            
            @Override
            public String getAccessibleDescription() {
                return AccessibleHTML.this.editor.getContentType();
            }
            
            @Override
            public AccessibleRole getAccessibleRole() {
                return AccessibleRole.TEXT;
            }
            
            @Override
            public int getIndexAtPoint(Point point) {
                View view = AccessibleHTML.TextElementInfo.this.getView();
                if (view != null) {
                    return view.viewToModel((float)point.x, (float)point.y, this.getBounds());
                }
                return -1;
            }
            
            @Override
            public Rectangle getCharacterBounds(int n) {
                try {
                    return AccessibleHTML.this.editor.getUI().modelToView(AccessibleHTML.this.editor, n);
                }
                catch (BadLocationException ex) {
                    return null;
                }
            }
            
            @Override
            public int getCharCount() {
                if (AccessibleHTML.TextElementInfo.this.validateIfNecessary()) {
                    Element element = this.elementInfo.getElement();
                    return element.getEndOffset() - element.getStartOffset();
                }
                return 0;
            }
            
            @Override
            public int getCaretPosition() {
                View view = AccessibleHTML.TextElementInfo.this.getView();
                if (view == null) {
                    return -1;
                }
                Container container = view.getContainer();
                if (container == null) {
                    return -1;
                }
                if ((container instanceof JTextComponent)) {
                    return ((JTextComponent)container).getCaretPosition();
                }
                return -1;
            }
            
            @Override
            public String getAtIndex(int n, int n2) {
                return this.getAtIndex(n, n2, 0);
            }
            
            @Override
            public String getAfterIndex(int n, int n2) {
                return this.getAtIndex(n, n2, 1);
            }
            
            @Override
            public String getBeforeIndex(int n, int n2) {
                return this.getAtIndex(n, n2, -1);
            }
            
            private String getAtIndex(int n, int n2, int n3) {
                if ((AccessibleHTML.this.model instanceof AbstractDocument)) {
                    ((AbstractDocument)AccessibleHTML.this.model).readLock();
                }
                try {
                    if (n2 < 0 || n2 >= AccessibleHTML.this.model.getLength()) {
                        __Null _Null = null;
                    }
                    else {
                        switch (n) {
                            case 1: {
                                if (n2 + n3 < AccessibleHTML.this.model.getLength() && n2 + n3 >= 0) {
                                    String text = AccessibleHTML.this.model.getText(n2 + n3, 1);
                                }
                                else {
                                    break;
                                }
                            }
                            case 2:
                            case 3: {
                                AccessibleHTML.TextElementInfo.TextAccessibleContext.IndexedSegment indexedSegment = this.getSegmentAt(n, n2);
                                if (indexedSegment != null) {
                                    if (n3 != 0) {
                                        int n4;
                                        if (n3 < 0) {
                                            n4 = indexedSegment.modelOffset - 1;
                                        }
                                        else {
                                            n4 = indexedSegment.modelOffset + n3 * indexedSegment.count;
                                        }
                                        if (n4 >= 0 && n4 <= AccessibleHTML.this.model.getLength()) {
                                            indexedSegment = this.getSegmentAt(n, n4);
                                        }
                                        else {
                                            indexedSegment = null;
                                        }
                                    }
                                    if (indexedSegment != null) {
                                        String o = new String(indexedSegment.array, indexedSegment.offset, indexedSegment.count);
                                    }
                                    else {
                                        break;
                                    }
                                }
                                else {
                                    break;
                                }
                            }
                        }
                    }
                }
                catch (BadLocationException ex) {}
                finally {
                    if ((AccessibleHTML.this.model instanceof AbstractDocument)) {
                        ((AbstractDocument)AccessibleHTML.this.model).readUnlock();
                    }
                }
                return null;
            }
            
            private Element getParagraphElement(int pos) {
                if ((AccessibleHTML.this.model instanceof PlainDocument)) {
                    return ((PlainDocument)AccessibleHTML.this.model).getParagraphElement(pos);
                }
                if ((AccessibleHTML.this.model instanceof StyledDocument)) {
                    return ((StyledDocument)AccessibleHTML.this.model).getParagraphElement(pos);
                }
                Element element;
                for (element = AccessibleHTML.this.model.getDefaultRootElement(); !element.isLeaf(); element = element.getElement(element.getElementIndex(pos))) {}
                if (element == null) {
                    return null;
                }
                return element.getParentElement();
            }
            
            private AccessibleHTML.TextElementInfo.TextAccessibleContext.IndexedSegment getParagraphElementText(int n) throws BadLocationException {
                Element paragraphElement = this.getParagraphElement(n);
                if (paragraphElement != null) {
                    AccessibleHTML.TextElementInfo.TextAccessibleContext.IndexedSegment indexedSegment = new AccessibleHTML.TextElementInfo.TextAccessibleContext.IndexedSegment();
                    try {
                        AccessibleHTML.this.model.getText(paragraphElement.getStartOffset(), paragraphElement.getEndOffset() - paragraphElement.getStartOffset(), indexedSegment);
                    }
                    catch (BadLocationException ex) {
                        return null;
                    }
                    indexedSegment.modelOffset = paragraphElement.getStartOffset();
                    return indexedSegment;
                }
                return null;
            }
            
            private AccessibleHTML.TextElementInfo.TextAccessibleContext.IndexedSegment getSegmentAt(int n, int n2) throws BadLocationException {
                AccessibleHTML.TextElementInfo.TextAccessibleContext.IndexedSegment paragraphElementText = this.getParagraphElementText(n2);
                if (paragraphElementText == null) {
                    return null;
                }
                BreakIterator breakIterator;
                switch (n) {
                    case 2: {
                        breakIterator = BreakIterator.getWordInstance(this.getLocale());
                        break;
                    }
                    case 3: {
                        breakIterator = BreakIterator.getSentenceInstance(this.getLocale());
                        break;
                    }
                    default: {
                        return null;
                    }
                }
                paragraphElementText.first();
                breakIterator.setText(paragraphElementText);
                int following = breakIterator.following(n2 - paragraphElementText.modelOffset + paragraphElementText.offset);
                if (following == -1) {
                    return null;
                }
                if (following > paragraphElementText.offset + paragraphElementText.count) {
                    return null;
                }
                int previous = breakIterator.previous();
                if (previous == -1 || previous >= paragraphElementText.offset + paragraphElementText.count) {
                    return null;
                }
                paragraphElementText.modelOffset = paragraphElementText.modelOffset + previous - paragraphElementText.offset;
                paragraphElementText.offset = previous;
                paragraphElementText.count = following - previous;
                return paragraphElementText;
            }
            
            @Override
            public AttributeSet getCharacterAttribute(int n) {
                if ((AccessibleHTML.this.model instanceof StyledDocument)) {
                    Element characterElement = ((StyledDocument)AccessibleHTML.this.model).getCharacterElement(n);
                    if (characterElement != null) {
                        return characterElement.getAttributes();
                    }
                }
                return null;
            }
            
            @Override
            public int getSelectionStart() {
                return AccessibleHTML.this.editor.getSelectionStart();
            }
            
            @Override
            public int getSelectionEnd() {
                return AccessibleHTML.this.editor.getSelectionEnd();
            }
            
            @Override
            public String getSelectedText() {
                return AccessibleHTML.this.editor.getSelectedText();
            }
            
            private String getText(int n, int n2) throws BadLocationException {
                if (AccessibleHTML.this.model != null && AccessibleHTML.this.model instanceof StyledDocument) {
                    StyledDocument styledDocument = (StyledDocument)AccessibleHTML.this.model;
                    return AccessibleHTML.this.model.getText(n, n2);
                }
                return null;
            }
            
            private class IndexedSegment extends Segment
            {
                public int modelOffset;
            }
        }
    }
    
    private class IconElementInfo extends AccessibleHTML.ElementInfo implements Accessible
    {
        private int width;
        private int height;
        private AccessibleContext accessibleContext;
        
        IconElementInfo(Element element, AccessibleHTML.ElementInfo elementInfo) {
            super(element, elementInfo);
            this.width = -1;
            this.height = -1;
        }
        
        @Override
        protected void invalidate(boolean b) {
            super.invalidate(b);
            int n = -1;
            this.height = n;
            this.width = n;
        }
        
        private int getImageSize(Object o) {
            if (this.validateIfNecessary()) {
                int n = this.getIntAttr(this.getAttributes(), o, -1);
                if (n == -1) {
                    View view = this.getView();
                    n = 0;
                    if ((view instanceof ImageView)) {
                        Image image = ((ImageView)view).getImage();
                        if (image != null) {
                            if (o == HTML.Attribute.WIDTH) {
                                n = image.getWidth(null);
                            }
                            else {
                                n = image.getHeight(null);
                            }
                        }
                    }
                }
                return n;
            }
            else {
                return 0;
            }
        }
        
        @Override
        public AccessibleContext getAccessibleContext() {
            if (this.accessibleContext == null) {
                this.accessibleContext = new AccessibleHTML.IconElementInfo.IconAccessibleContext(this);
            }
            return this.accessibleContext;
        }
        
        static int access$800(AccessibleHTML.IconElementInfo iconElementInfo, Object o) {
            return iconElementInfo.getImageSize(o);
        }
        
        protected class IconAccessibleContext extends AccessibleHTML.HTMLAccessibleContext implements AccessibleIcon
        {
            public IconAccessibleContext(AccessibleHTML.ElementInfo elementInfo) {
                AccessibleHTML.IconElementInfo.this.this$0.super(elementInfo);
            }
            
            @Override
            public String getAccessibleName() {
                return this.getAccessibleIconDescription();
            }
            
            @Override
            public String getAccessibleDescription() {
                return AccessibleHTML.this.editor.getContentType();
            }
            
            @Override
            public AccessibleRole getAccessibleRole() {
                return AccessibleRole.ICON;
            }
            
            @Override
            public AccessibleIcon[] getAccessibleIcon() {
                return new AccessibleIcon[] { this };
            }
            
            @Override
            public String getAccessibleIconDescription() {
                return ((ImageView)AccessibleHTML.IconElementInfo.this.getView()).getAltText();
            }
            
            @Override
            public void setAccessibleIconDescription(String s) {
            }
            
            @Override
            public int getAccessibleIconWidth() {
                if (AccessibleHTML.IconElementInfo.this.width == -1) {
                    AccessibleHTML.IconElementInfo.this.width = AccessibleHTML.IconElementInfo.access$800(AccessibleHTML.IconElementInfo.this, HTML.Attribute.WIDTH);
                }
                return AccessibleHTML.IconElementInfo.this.width;
            }
            
            @Override
            public int getAccessibleIconHeight() {
                if (AccessibleHTML.IconElementInfo.this.height == -1) {
                    AccessibleHTML.IconElementInfo.this.height = AccessibleHTML.IconElementInfo.access$800(AccessibleHTML.IconElementInfo.this, HTML.Attribute.HEIGHT);
                }
                return AccessibleHTML.IconElementInfo.this.height;
            }
        }
    }
    
    private class TableElementInfo extends AccessibleHTML.ElementInfo implements Accessible
    {
        protected AccessibleHTML.ElementInfo caption;
        private AccessibleHTML.TableElementInfo.TableCellElementInfo[][] grid;
        private AccessibleContext accessibleContext;
        
        TableElementInfo(Element element, AccessibleHTML.ElementInfo elementInfo) {
            super(element, elementInfo);
        }
        
        public AccessibleHTML.ElementInfo getCaptionInfo() {
            return this.caption;
        }
        
        @Override
        protected void validate() {
            super.validate();
            this.updateGrid();
        }
        
        @Override
        protected void loadChildren(Element element) {
            for (int i = 0; i < element.getElementCount(); ++i) {
                Element element2 = element.getElement(i);
                AttributeSet attributes = element2.getAttributes();
                if (attributes.getAttribute(StyleConstants.NameAttribute) == HTML.Tag.TR) {
                    this.addChild(new AccessibleHTML.TableElementInfo.TableRowElementInfo(element2, this, i));
                }
                else if (attributes.getAttribute(StyleConstants.NameAttribute) == HTML.Tag.CAPTION) {
                    this.caption = AccessibleHTML.this.createElementInfo(element2, this);
                }
            }
        }
        
        private void updateGrid() {
            int max = 0;
            int max2 = 0;
            AccessibleHTML.TableElementInfo.TableRowElementInfo row;
            int max3;
            for (int i = 0; i < this.getChildCount(); max = Math.max(row.getRowCount(), max), --max, max2 = Math.max(max2, row.getColumnCount() + max3), ++i) {
                row = this.getRow(i);
                max3 = 0;
                for (int j = 0; j < max; ++j) {
                    max3 = Math.max(max3, AccessibleHTML.TableElementInfo.TableRowElementInfo.access$1000(this.getRow(i - j - 1), j + 2));
                }
            }
            int n = this.getChildCount() + max;
            this.grid = new AccessibleHTML.TableElementInfo.TableCellElementInfo[n][];
            for (int k = 0; k < n; ++k) {
                this.grid[k] = new AccessibleHTML.TableElementInfo.TableCellElementInfo[max2];
            }
            for (int l = 0; l < n; ++l) {
                AccessibleHTML.TableElementInfo.TableRowElementInfo.access$1100(this.getRow(l), l);
            }
        }
        
        public AccessibleHTML.TableElementInfo.TableRowElementInfo getRow(int n) {
            return (AccessibleHTML.TableElementInfo.TableRowElementInfo)this.getChild(n);
        }
        
        public AccessibleHTML.TableElementInfo.TableCellElementInfo getCell(int n, int n2) {
            if (this.validateIfNecessary() && n < this.grid.length && n2 < this.grid[0].length) {
                return this.grid[n][n2];
            }
            return null;
        }
        
        public int getRowExtentAt(int n, int n2) {
            AccessibleHTML.TableElementInfo.TableCellElementInfo cell = this.getCell(n, n2);
            if (cell != null) {
                int rowCount = cell.getRowCount();
                int n3;
                for (n3 = 1; n - n3 >= 0 && this.grid[n - n3][n2] == cell; ++n3) {}
                return rowCount - n3 + 1;
            }
            return 0;
        }
        
        public int getColumnExtentAt(int n, int n2) {
            AccessibleHTML.TableElementInfo.TableCellElementInfo cell = this.getCell(n, n2);
            if (cell != null) {
                int columnCount = cell.getColumnCount();
                int n3;
                for (n3 = 1; n2 - n3 >= 0 && this.grid[n][n2 - n3] == cell; ++n3) {}
                return columnCount - n3 + 1;
            }
            return 0;
        }
        
        public int getRowCount() {
            if (this.validateIfNecessary()) {
                return this.grid.length;
            }
            return 0;
        }
        
        public int getColumnCount() {
            if (this.validateIfNecessary() && this.grid.length > 0) {
                return this.grid[0].length;
            }
            return 0;
        }
        
        @Override
        public AccessibleContext getAccessibleContext() {
            if (this.accessibleContext == null) {
                this.accessibleContext = new AccessibleHTML.TableElementInfo.TableAccessibleContext(this);
            }
            return this.accessibleContext;
        }
        
        public class TableAccessibleContext extends AccessibleHTML.HTMLAccessibleContext implements AccessibleTable
        {
            private AccessibleHTML.TableElementInfo.TableAccessibleContext.AccessibleHeadersTable rowHeadersTable;
            
            public TableAccessibleContext(AccessibleHTML.ElementInfo elementInfo) {
                AccessibleHTML.TableElementInfo.this.this$0.super(elementInfo);
            }
            
            @Override
            public String getAccessibleName() {
                return this.getAccessibleRole().toString();
            }
            
            @Override
            public String getAccessibleDescription() {
                return AccessibleHTML.this.editor.getContentType();
            }
            
            @Override
            public AccessibleRole getAccessibleRole() {
                return AccessibleRole.TABLE;
            }
            
            @Override
            public int getAccessibleIndexInParent() {
                return this.elementInfo.getIndexInParent();
            }
            
            @Override
            public int getAccessibleChildrenCount() {
                return ((AccessibleHTML.TableElementInfo)this.elementInfo).getRowCount() * ((AccessibleHTML.TableElementInfo)this.elementInfo).getColumnCount();
            }
            
            @Override
            public Accessible getAccessibleChild(int n) {
                int rowCount = ((AccessibleHTML.TableElementInfo)this.elementInfo).getRowCount();
                int columnCount = ((AccessibleHTML.TableElementInfo)this.elementInfo).getColumnCount();
                int n2 = n / rowCount;
                int n3 = n % columnCount;
                if (n2 < 0 || n2 >= rowCount || n3 < 0 || n3 >= columnCount) {
                    return null;
                }
                return this.getAccessibleAt(n2, n3);
            }
            
            @Override
            public AccessibleTable getAccessibleTable() {
                return this;
            }
            
            @Override
            public Accessible getAccessibleCaption() {
                if ((AccessibleHTML.TableElementInfo.this.getCaptionInfo() instanceof Accessible)) {
                    return (Accessible)AccessibleHTML.TableElementInfo.this.caption;
                }
                return null;
            }
            
            @Override
            public void setAccessibleCaption(Accessible accessible) {
            }
            
            @Override
            public Accessible getAccessibleSummary() {
                return null;
            }
            
            @Override
            public void setAccessibleSummary(Accessible accessible) {
            }
            
            @Override
            public int getAccessibleRowCount() {
                return ((AccessibleHTML.TableElementInfo)this.elementInfo).getRowCount();
            }
            
            @Override
            public int getAccessibleColumnCount() {
                return ((AccessibleHTML.TableElementInfo)this.elementInfo).getColumnCount();
            }
            
            @Override
            public Accessible getAccessibleAt(int n, int n2) {
                AccessibleHTML.TableElementInfo.TableCellElementInfo cell = AccessibleHTML.TableElementInfo.this.getCell(n, n2);
                if (cell != null) {
                    return cell.getAccessible();
                }
                return null;
            }
            
            @Override
            public int getAccessibleRowExtentAt(int n, int n2) {
                return ((AccessibleHTML.TableElementInfo)this.elementInfo).getRowExtentAt(n, n2);
            }
            
            @Override
            public int getAccessibleColumnExtentAt(int n, int n2) {
                return ((AccessibleHTML.TableElementInfo)this.elementInfo).getColumnExtentAt(n, n2);
            }
            
            @Override
            public AccessibleTable getAccessibleRowHeader() {
                return this.rowHeadersTable;
            }
            
            @Override
            public void setAccessibleRowHeader(AccessibleTable accessibleTable) {
            }
            
            @Override
            public AccessibleTable getAccessibleColumnHeader() {
                return null;
            }
            
            @Override
            public void setAccessibleColumnHeader(AccessibleTable accessibleTable) {
            }
            
            @Override
            public Accessible getAccessibleRowDescription(int n) {
                return null;
            }
            
            @Override
            public void setAccessibleRowDescription(int n, Accessible accessible) {
            }
            
            @Override
            public Accessible getAccessibleColumnDescription(int n) {
                return null;
            }
            
            @Override
            public void setAccessibleColumnDescription(int n, Accessible accessible) {
            }
            
            @Override
            public boolean isAccessibleSelected(int n, int n2) {
                if (AccessibleHTML.TableElementInfo.this.validateIfNecessary()) {
                    if (n < 0 || n >= this.getAccessibleRowCount() || n2 < 0 || n2 >= this.getAccessibleColumnCount()) {
                        return false;
                    }
                    AccessibleHTML.TableElementInfo.TableCellElementInfo cell = AccessibleHTML.TableElementInfo.this.getCell(n, n2);
                    if (cell != null) {
                        Element element = cell.getElement();
                        int startOffset = element.getStartOffset();
                        int endOffset = element.getEndOffset();
                        return startOffset >= AccessibleHTML.this.editor.getSelectionStart() && endOffset <= AccessibleHTML.this.editor.getSelectionEnd();
                    }
                }
                return false;
            }
            
            @Override
            public boolean isAccessibleRowSelected(int n) {
                if (AccessibleHTML.TableElementInfo.this.validateIfNecessary()) {
                    if (n < 0 || n >= this.getAccessibleRowCount()) {
                        return false;
                    }
                    int accessibleColumnCount = this.getAccessibleColumnCount();
                    AccessibleHTML.TableElementInfo.TableCellElementInfo cell = AccessibleHTML.TableElementInfo.this.getCell(n, 0);
                    if (cell == null) {
                        return false;
                    }
                    int startOffset = cell.getElement().getStartOffset();
                    AccessibleHTML.TableElementInfo.TableCellElementInfo cell2 = AccessibleHTML.TableElementInfo.this.getCell(n, accessibleColumnCount - 1);
                    if (cell2 == null) {
                        return false;
                    }
                    int endOffset = cell2.getElement().getEndOffset();
                    return startOffset >= AccessibleHTML.this.editor.getSelectionStart() && endOffset <= AccessibleHTML.this.editor.getSelectionEnd();
                }
                else {
                    return false;
                }
            }
            
            @Override
            public boolean isAccessibleColumnSelected(int n) {
                if (AccessibleHTML.TableElementInfo.this.validateIfNecessary()) {
                    if (n < 0 || n >= this.getAccessibleColumnCount()) {
                        return false;
                    }
                    int accessibleRowCount = this.getAccessibleRowCount();
                    AccessibleHTML.TableElementInfo.TableCellElementInfo cell = AccessibleHTML.TableElementInfo.this.getCell(0, n);
                    if (cell == null) {
                        return false;
                    }
                    int startOffset = cell.getElement().getStartOffset();
                    AccessibleHTML.TableElementInfo.TableCellElementInfo cell2 = AccessibleHTML.TableElementInfo.this.getCell(accessibleRowCount - 1, n);
                    if (cell2 == null) {
                        return false;
                    }
                    int endOffset = cell2.getElement().getEndOffset();
                    return startOffset >= AccessibleHTML.this.editor.getSelectionStart() && endOffset <= AccessibleHTML.this.editor.getSelectionEnd();
                }
                else {
                    return false;
                }
            }
            
            @Override
            public int[] getSelectedAccessibleRows() {
                if (AccessibleHTML.TableElementInfo.this.validateIfNecessary()) {
                    int accessibleRowCount = this.getAccessibleRowCount();
                    Vector<Integer> vector = new Vector();
                    for (int i = 0; i < accessibleRowCount; ++i) {
                        if (this.isAccessibleRowSelected(i)) {
                            vector.addElement(Integer.valueOf(i));
                        }
                    }
                    int[] array = new int[vector.size()];
                    for (int j = 0; j < array.length; ++j) {
                        array[j] = (int)vector.elementAt(j);
                    }
                    return array;
                }
                return new int[0];
            }
            
            @Override
            public int[] getSelectedAccessibleColumns() {
                if (AccessibleHTML.TableElementInfo.this.validateIfNecessary()) {
                    int accessibleRowCount = this.getAccessibleRowCount();
                    Vector<Integer> vector = new Vector();
                    for (int i = 0; i < accessibleRowCount; ++i) {
                        if (this.isAccessibleColumnSelected(i)) {
                            vector.addElement(Integer.valueOf(i));
                        }
                    }
                    int[] array = new int[vector.size()];
                    for (int j = 0; j < array.length; ++j) {
                        array[j] = (int)vector.elementAt(j);
                    }
                    return array;
                }
                return new int[0];
            }
            
            public int getAccessibleRow(int n) {
                if (AccessibleHTML.TableElementInfo.this.validateIfNecessary()) {
                    if (n >= this.getAccessibleColumnCount() * this.getAccessibleRowCount()) {
                        return -1;
                    }
                    return n / this.getAccessibleColumnCount();
                }
                else {
                    return -1;
                }
            }
            
            public int getAccessibleColumn(int n) {
                if (AccessibleHTML.TableElementInfo.this.validateIfNecessary()) {
                    if (n >= this.getAccessibleColumnCount() * this.getAccessibleRowCount()) {
                        return -1;
                    }
                    return n % this.getAccessibleColumnCount();
                }
                else {
                    return -1;
                }
            }
            
            public int getAccessibleIndex(int n, int n2) {
                if (AccessibleHTML.TableElementInfo.this.validateIfNecessary()) {
                    if (n >= this.getAccessibleRowCount() || n2 >= this.getAccessibleColumnCount()) {
                        return -1;
                    }
                    return n * this.getAccessibleColumnCount() + n2;
                }
                else {
                    return -1;
                }
            }
            
            public String getAccessibleRowHeader(int n) {
                if (AccessibleHTML.TableElementInfo.this.validateIfNecessary()) {
                    AccessibleHTML.TableElementInfo.TableCellElementInfo cell = AccessibleHTML.TableElementInfo.this.getCell(n, 0);
                    if (cell.isHeaderCell()) {
                        View view = cell.getView();
                        if (view != null && AccessibleHTML.this.model != null) {
                            try {
                                return AccessibleHTML.this.model.getText(view.getStartOffset(), view.getEndOffset() - view.getStartOffset());
                            }
                            catch (BadLocationException ex) {
                                return null;
                            }
                        }
                    }
                }
                return null;
            }
            
            public String getAccessibleColumnHeader(int n) {
                if (AccessibleHTML.TableElementInfo.this.validateIfNecessary()) {
                    AccessibleHTML.TableElementInfo.TableCellElementInfo cell = AccessibleHTML.TableElementInfo.this.getCell(0, n);
                    if (cell.isHeaderCell()) {
                        View view = cell.getView();
                        if (view != null && AccessibleHTML.this.model != null) {
                            try {
                                return AccessibleHTML.this.model.getText(view.getStartOffset(), view.getEndOffset() - view.getStartOffset());
                            }
                            catch (BadLocationException ex) {
                                return null;
                            }
                        }
                    }
                }
                return null;
            }
            
            public void addRowHeader(AccessibleHTML.TableElementInfo.TableCellElementInfo tableCellElementInfo, int n) {
                if (this.rowHeadersTable == null) {
                    this.rowHeadersTable = new AccessibleHTML.TableElementInfo.TableAccessibleContext.AccessibleHeadersTable();
                }
                this.rowHeadersTable.addHeader(tableCellElementInfo, n);
            }
            
            protected class AccessibleHeadersTable implements AccessibleTable
            {
                private Hashtable<Integer, ArrayList<AccessibleHTML.TableElementInfo.TableCellElementInfo>> headers;
                private int rowCount;
                private int columnCount;
                
                protected AccessibleHeadersTable() {
                    this.headers = new Hashtable();
                    this.rowCount = 0;
                    this.columnCount = 0;
                }
                
                public void addHeader(AccessibleHTML.TableElementInfo.TableCellElementInfo e, int i) {
                    Integer value = Integer.valueOf(i);
                    ArrayList<AccessibleHTML.TableElementInfo.TableCellElementInfo> value2 = (ArrayList)this.headers.get(value);
                    if (value2 == null) {
                        value2 = new ArrayList();
                        this.headers.put(value, value2);
                    }
                    value2.add(e);
                }
                
                @Override
                public Accessible getAccessibleCaption() {
                    return null;
                }
                
                @Override
                public void setAccessibleCaption(Accessible accessible) {
                }
                
                @Override
                public Accessible getAccessibleSummary() {
                    return null;
                }
                
                @Override
                public void setAccessibleSummary(Accessible accessible) {
                }
                
                @Override
                public int getAccessibleRowCount() {
                    return this.rowCount;
                }
                
                @Override
                public int getAccessibleColumnCount() {
                    return this.columnCount;
                }
                
                private AccessibleHTML.TableElementInfo.TableCellElementInfo getElementInfoAt(int i, int index) {
                    ArrayList<Object> list = (ArrayList)this.headers.get(Integer.valueOf(i));
                    if (list != null) {
                        return (AccessibleHTML.TableElementInfo.TableCellElementInfo)list.get(index);
                    }
                    return null;
                }
                
                @Override
                public Accessible getAccessibleAt(int n, int n2) {
                    AccessibleHTML.TableElementInfo.TableCellElementInfo elementInfo = this.getElementInfoAt(n, n2);
                    if ((elementInfo instanceof Accessible)) {
                        return (Accessible)elementInfo;
                    }
                    return null;
                }
                
                @Override
                public int getAccessibleRowExtentAt(int n, int n2) {
                    AccessibleHTML.TableElementInfo.TableCellElementInfo elementInfo = this.getElementInfoAt(n, n2);
                    if (elementInfo != null) {
                        return elementInfo.getRowCount();
                    }
                    return 0;
                }
                
                @Override
                public int getAccessibleColumnExtentAt(int n, int n2) {
                    AccessibleHTML.TableElementInfo.TableCellElementInfo elementInfo = this.getElementInfoAt(n, n2);
                    if (elementInfo != null) {
                        return elementInfo.getRowCount();
                    }
                    return 0;
                }
                
                @Override
                public AccessibleTable getAccessibleRowHeader() {
                    return null;
                }
                
                @Override
                public void setAccessibleRowHeader(AccessibleTable accessibleTable) {
                }
                
                @Override
                public AccessibleTable getAccessibleColumnHeader() {
                    return null;
                }
                
                @Override
                public void setAccessibleColumnHeader(AccessibleTable accessibleTable) {
                }
                
                @Override
                public Accessible getAccessibleRowDescription(int n) {
                    return null;
                }
                
                @Override
                public void setAccessibleRowDescription(int n, Accessible accessible) {
                }
                
                @Override
                public Accessible getAccessibleColumnDescription(int n) {
                    return null;
                }
                
                @Override
                public void setAccessibleColumnDescription(int n, Accessible accessible) {
                }
                
                @Override
                public boolean isAccessibleSelected(int n, int n2) {
                    return false;
                }
                
                @Override
                public boolean isAccessibleRowSelected(int n) {
                    return false;
                }
                
                @Override
                public boolean isAccessibleColumnSelected(int n) {
                    return false;
                }
                
                @Override
                public int[] getSelectedAccessibleRows() {
                    return new int[0];
                }
                
                @Override
                public int[] getSelectedAccessibleColumns() {
                    return new int[0];
                }
            }
        }
        
        private class TableRowElementInfo extends AccessibleHTML.ElementInfo
        {
            private AccessibleHTML.TableElementInfo parent;
            private int rowNumber;
            
            TableRowElementInfo(Element element, AccessibleHTML.TableElementInfo parent, int rowNumber) {
                AccessibleHTML.TableElementInfo.this.this$0.super(element, parent);
                this.parent = parent;
                this.rowNumber = rowNumber;
            }
            
            @Override
            protected void loadChildren(Element element) {
                for (int i = 0; i < element.getElementCount(); ++i) {
                    AttributeSet attributes = element.getElement(i).getAttributes();
                    if (attributes.getAttribute(StyleConstants.NameAttribute) == HTML.Tag.TH) {
                        AccessibleHTML.TableElementInfo.TableCellElementInfo o = new AccessibleHTML.TableElementInfo.TableCellElementInfo(element.getElement(i), this, true);
                        this.addChild(o);
                        ((AccessibleHTML.TableElementInfo.TableAccessibleContext)this.parent.getAccessibleContext().getAccessibleTable()).addRowHeader(o, this.rowNumber);
                    }
                    else if (attributes.getAttribute(StyleConstants.NameAttribute) == HTML.Tag.TD) {
                        this.addChild(new AccessibleHTML.TableElementInfo.TableCellElementInfo(element.getElement(i), this, false));
                    }
                }
            }
            
            public int getRowCount() {
                int max = 1;
                if (this.validateIfNecessary()) {
                    for (int i = 0; i < this.getChildCount(); ++i) {
                        AccessibleHTML.TableElementInfo.TableCellElementInfo tableCellElementInfo = (AccessibleHTML.TableElementInfo.TableCellElementInfo)this.getChild(i);
                        if (tableCellElementInfo.validateIfNecessary()) {
                            max = Math.max(max, tableCellElementInfo.getRowCount());
                        }
                    }
                }
                return max;
            }
            
            public int getColumnCount() {
                int n = 0;
                if (this.validateIfNecessary()) {
                    for (int i = 0; i < this.getChildCount(); ++i) {
                        AccessibleHTML.TableElementInfo.TableCellElementInfo tableCellElementInfo = (AccessibleHTML.TableElementInfo.TableCellElementInfo)this.getChild(i);
                        if (tableCellElementInfo.validateIfNecessary()) {
                            n += tableCellElementInfo.getColumnCount();
                        }
                    }
                }
                return n;
            }
            
            @Override
            protected void invalidate(boolean b) {
                super.invalidate(b);
                this.getParent().invalidate(true);
            }
            
            private void updateGrid(int n) {
                if (this.validateIfNecessary()) {
                    int i = 0;
                    while (i == 0) {
                        int j = 0;
                        while (j < AccessibleHTML.TableElementInfo.this.grid[n].length) {
                            if (AccessibleHTML.TableElementInfo.this.grid[n][j] == null) {
                                i = 1;
                                break;
                            }
                            ++j;
                        }
                        if (i != 0) {
                            continue;
                        }
                        ++n;
                    }
                    AccessibleHTML.TableElementInfo.TableCellElementInfo tableCellElementInfo;
                    for (int n2 = 0, k = 0; k < this.getChildCount(); n2 += tableCellElementInfo.getColumnCount(), ++k) {
                        tableCellElementInfo = (AccessibleHTML.TableElementInfo.TableCellElementInfo)this.getChild(k);
                        while (AccessibleHTML.TableElementInfo.this.grid[n][n2] != null) {
                            ++n2;
                        }
                        for (int l = tableCellElementInfo.getRowCount() - 1; l >= 0; --l) {
                            for (int n3 = tableCellElementInfo.getColumnCount() - 1; n3 >= 0; --n3) {
                                AccessibleHTML.TableElementInfo.this.grid[n + l][n2 + n3] = tableCellElementInfo;
                            }
                        }
                    }
                }
            }
            
            private int getColumnCount(int n) {
                if (this.validateIfNecessary()) {
                    int n2 = 0;
                    for (int i = 0; i < this.getChildCount(); ++i) {
                        AccessibleHTML.TableElementInfo.TableCellElementInfo tableCellElementInfo = (AccessibleHTML.TableElementInfo.TableCellElementInfo)this.getChild(i);
                        if (tableCellElementInfo.getRowCount() >= n) {
                            n2 += tableCellElementInfo.getColumnCount();
                        }
                    }
                    return n2;
                }
                return 0;
            }
            
            static int access$1000(AccessibleHTML.TableElementInfo.TableRowElementInfo tableRowElementInfo, int n) {
                return tableRowElementInfo.getColumnCount(n);
            }
            
            static void access$1100(AccessibleHTML.TableElementInfo.TableRowElementInfo tableRowElementInfo, int n) {
                tableRowElementInfo.updateGrid(n);
            }
        }
        
        private class TableCellElementInfo extends AccessibleHTML.ElementInfo
        {
            private Accessible accessible;
            private boolean isHeaderCell;
            
            TableCellElementInfo(Element element, AccessibleHTML.ElementInfo elementInfo) {
                AccessibleHTML.TableElementInfo.this.this$0.super(element, elementInfo);
                this.isHeaderCell = false;
            }
            
            TableCellElementInfo(Element element, AccessibleHTML.ElementInfo elementInfo, boolean isHeaderCell) {
                AccessibleHTML.TableElementInfo.this.this$0.super(element, elementInfo);
                this.isHeaderCell = isHeaderCell;
            }
            
            public boolean isHeaderCell() {
                return this.isHeaderCell;
            }
            
            public Accessible getAccessible() {
                this.accessible = null;
                this.getAccessible(this);
                return this.accessible;
            }
            
            private void getAccessible(AccessibleHTML.ElementInfo elementInfo) {
                if ((elementInfo instanceof Accessible)) {
                    this.accessible = (Accessible)elementInfo;
                }
                else {
                    for (int i = 0; i < elementInfo.getChildCount(); ++i) {
                        this.getAccessible(elementInfo.getChild(i));
                    }
                }
            }
            
            public int getRowCount() {
                if (this.validateIfNecessary()) {
                    return Math.max(1, this.getIntAttr(this.getAttributes(), HTML.Attribute.ROWSPAN, 1));
                }
                return 0;
            }
            
            public int getColumnCount() {
                if (this.validateIfNecessary()) {
                    return Math.max(1, this.getIntAttr(this.getAttributes(), HTML.Attribute.COLSPAN, 1));
                }
                return 0;
            }
            
            @Override
            protected void invalidate(boolean b) {
                super.invalidate(b);
                this.getParent().invalidate(true);
            }
        }
    }
    
    private class ElementInfo
    {
        private ArrayList<AccessibleHTML.ElementInfo> children;
        private Element element;
        private AccessibleHTML.ElementInfo parent;
        private boolean isValid;
        private boolean canBeValid;
        
        ElementInfo(AccessibleHTML accessibleHTML, Element element) {
            this(accessibleHTML, element, null);
        }
        
        ElementInfo(Element element, AccessibleHTML.ElementInfo parent) {
            this.element = element;
            this.parent = parent;
            this.isValid = false;
            this.canBeValid = true;
        }
        
        protected void validate() {
            this.isValid = true;
            this.loadChildren(this.getElement());
        }
        
        protected void loadChildren(Element element) {
            if (!element.isLeaf()) {
                for (int i = 0; i < element.getElementCount(); ++i) {
                    Element element2 = element.getElement(i);
                    AccessibleHTML.ElementInfo elementInfo = AccessibleHTML.this.createElementInfo(element2, this);
                    if (elementInfo != null) {
                        this.addChild(elementInfo);
                    }
                    else {
                        this.loadChildren(element2);
                    }
                }
            }
        }
        
        public int getIndexInParent() {
            if (this.parent == null || !this.parent.isValid()) {
                return -1;
            }
            return this.parent.indexOf(this);
        }
        
        public Element getElement() {
            return this.element;
        }
        
        public AccessibleHTML.ElementInfo getParent() {
            return this.parent;
        }
        
        public int indexOf(AccessibleHTML.ElementInfo o) {
            ArrayList<Object> children = this.children;
            if (children != null) {
                return children.indexOf(o);
            }
            return -1;
        }
        
        public AccessibleHTML.ElementInfo getChild(int index) {
            if (this.validateIfNecessary()) {
                ArrayList<Object> children = this.children;
                if (children != null && index >= 0 && index < children.size()) {
                    return (AccessibleHTML.ElementInfo)children.get(index);
                }
            }
            return null;
        }
        
        public int getChildCount() {
            this.validateIfNecessary();
            return (this.children == null) ? 0 : this.children.size();
        }
        
        protected void addChild(AccessibleHTML.ElementInfo e) {
            if (this.children == null) {
                this.children = new ArrayList();
            }
            this.children.add(e);
        }
        
        protected View getView() {
            if (!this.validateIfNecessary()) {
                return null;
            }
            Object access$1300 = AccessibleHTML.access$1300(AccessibleHTML.this);
            try {
                View access$1301 = AccessibleHTML.access$1400(AccessibleHTML.this);
                Element element = this.getElement();
                int startOffset = element.getStartOffset();
                if (access$1301 != null) {
                    return this.getView(access$1301, element, startOffset);
                }
                return null;
            }
            finally {
                AccessibleHTML.access$1500(AccessibleHTML.this, access$1300);
            }
        }
        
        public Rectangle getBounds() {
            if (!this.validateIfNecessary()) {
                return null;
            }
            Object access$1300 = AccessibleHTML.access$1300(AccessibleHTML.this);
            try {
                Rectangle access$1301 = AccessibleHTML.access$1600(AccessibleHTML.this);
                View access$1302 = AccessibleHTML.access$1400(AccessibleHTML.this);
                Element element = this.getElement();
                if (access$1301 != null && access$1302 != null) {
                    try {
                        return access$1302.modelToView(element.getStartOffset(), Position.Bias.Forward, element.getEndOffset(), Position.Bias.Backward, access$1301).getBounds();
                    }
                    catch (BadLocationException ex) {}
                }
            }
            finally {
                AccessibleHTML.access$1500(AccessibleHTML.this, access$1300);
            }
            return null;
        }
        
        protected boolean isValid() {
            return this.isValid;
        }
        
        protected AttributeSet getAttributes() {
            if (this.validateIfNecessary()) {
                return this.getElement().getAttributes();
            }
            return null;
        }
        
        protected AttributeSet getViewAttributes() {
            if (this.validateIfNecessary()) {
                View view = this.getView();
                if (view != null) {
                    return view.getElement().getAttributes();
                }
                return this.getElement().getAttributes();
            }
            else {
                return null;
            }
        }
        
        protected int getIntAttr(AttributeSet set, Object o, int max) {
            if (set != null && set.isDefined(o)) {
                String s = (String)set.getAttribute(o);
                if (s == null) {
                }
                else {
                    try {
                        max = Math.max(0, Integer.parseInt(s));
                    }
                    catch (NumberFormatException ex) {
                    }
                }
                return max;
            }
            else {
                return max;
            }
        }
        
        protected boolean validateIfNecessary() {
            if (!this.isValid() && this.canBeValid) {
                this.children = null;
                Object access$1300 = AccessibleHTML.access$1300(AccessibleHTML.this);
                try {
                    this.validate();
                }
                finally {
                    AccessibleHTML.access$1500(AccessibleHTML.this, access$1300);
                }
            }
            return this.isValid();
        }
        
        protected void invalidate(boolean canBeValid) {
            if (!this.isValid()) {
                if (this.canBeValid && !canBeValid) {
                    this.canBeValid = false;
                }
                return;
            }
            else {
                this.isValid = false;
                this.canBeValid = canBeValid;
                if (this.children != null) {
                    Iterator<AccessibleHTML.ElementInfo> iterator = this.children.iterator();
                    while (iterator.hasNext()) {
                        ((AccessibleHTML.ElementInfo)iterator.next()).invalidate(false);
                    }
                    this.children = null;
                }
                return;
            }
        }
        
        private View getView(View view, Element element, int pos) {
            if (view.getElement() == element) {
                return view;
            }
            int viewIndex = view.getViewIndex(pos, Position.Bias.Forward);
            if (viewIndex != -1 && viewIndex < view.getViewCount()) {
                return this.getView(view.getView(viewIndex), element, pos);
            }
            return null;
        }
        
        private int getClosestInfoIndex(int n) {
            int i = 0;
            while (i < this.getChildCount()) {
                AccessibleHTML.ElementInfo child = this.getChild(i);
                if (n < child.getElement().getEndOffset() || n == child.getElement().getStartOffset()) {
                    return i;
                }
                ++i;
            }
            return -1;
        }
        
        private void update(DocumentEvent documentEvent) {
            if (!this.isValid()) {
                return;
            }
            AccessibleHTML.ElementInfo parent = this.getParent();
            Element element = this.getElement();
            while (documentEvent.getChange(element) == null) {
                element = element.getParentElement();
                if (parent != null && element != null && element != parent.getElement()) {
                    continue;
                }
                if (this.getChildCount() > 0) {
                    Element element2 = this.getElement();
                    int offset = documentEvent.getOffset();
                    int a = this.getClosestInfoIndex(offset);
                    if (a == -1 && documentEvent.getType() == DocumentEvent.EventType.REMOVE && offset >= element2.getEndOffset()) {
                        a = this.getChildCount() - 1;
                    }
                    AccessibleHTML.ElementInfo elementInfo = (a >= 0) ? this.getChild(a) : null;
                    if (elementInfo != null && elementInfo.getElement().getStartOffset() == offset && offset > 0) {
                        a = Math.max(a - 1, 0);
                    }
                    int closestInfoIndex;
                    if (documentEvent.getType() != DocumentEvent.EventType.REMOVE) {
                        closestInfoIndex = this.getClosestInfoIndex(offset + documentEvent.getLength());
                        if (closestInfoIndex < 0) {
                            closestInfoIndex = this.getChildCount() - 1;
                        }
                    }
                    else {
                        for (closestInfoIndex = a; closestInfoIndex + 1 < this.getChildCount() && this.getChild(closestInfoIndex + 1).getElement().getEndOffset() == this.getChild(closestInfoIndex + 1).getElement().getStartOffset(); ++closestInfoIndex) {}
                    }
                    for (int max = Math.max(a, 0); max <= closestInfoIndex && this.isValid(); ++max) {
                        this.getChild(max).update(documentEvent);
                    }
                }
                return;
            }
            if (element == this.getElement()) {
                this.invalidate(true);
            }
            else if (parent != null) {
                parent.invalidate(parent == AccessibleHTML.access$1700(AccessibleHTML.this));
            }
            return;
        }
        
        static void access$1800(AccessibleHTML.ElementInfo elementInfo, DocumentEvent documentEvent) {
            elementInfo.update(documentEvent);
        }
    }
    
    private class DocumentHandler implements DocumentListener
    {
        @Override
        public void insertUpdate(DocumentEvent documentEvent) {
            AccessibleHTML.ElementInfo.access$1800(AccessibleHTML.access$1700(AccessibleHTML.this), documentEvent);
        }
        
        @Override
        public void removeUpdate(DocumentEvent documentEvent) {
            AccessibleHTML.ElementInfo.access$1800(AccessibleHTML.access$1700(AccessibleHTML.this), documentEvent);
        }
        
        @Override
        public void changedUpdate(DocumentEvent documentEvent) {
            AccessibleHTML.ElementInfo.access$1800(AccessibleHTML.access$1700(AccessibleHTML.this), documentEvent);
        }
    }
    
    private class PropertyChangeHandler implements PropertyChangeListener
    {
        @Override
        public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
            if (propertyChangeEvent.getPropertyName().equals("document")) {
                AccessibleHTML.access$1900(AccessibleHTML.this, AccessibleHTML.this.editor.getDocument());
            }
        }
    }
}
